# Minimum required module file
# ----------------------------
# Because pynonymizer.version is imported by setup.py, this needs to stay empty, lest it cause import errors pre-install
# However, this makes sense because this is the base package for a tool, and we have other entry points anyway.
#
# So if you're thinking about adding application code here, you're probably in the wrong place.
